------------------------------------------------------------------------------
--                              G N A T C O L L                             --
--                                                                          --
--                       Copyright (C) 2021, AdaCore                        --
--                                                                          --
-- This library is free software;  you can redistribute it and/or modify it --
-- under terms of the  GNU General Public License  as published by the Free --
-- Software  Foundation;  either version 3,  or (at your  option) any later --
-- version. This library is distributed in the hope that it will be useful, --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE.                            --
--                                                                          --
-- As a special exception under Section 7 of GPL version 3, you are granted --
-- additional permissions described in the GCC Runtime Library Exception,   --
-- version 3.1, as published by the Free Software Foundation.               --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
------------------------------------------------------------------------------

with GNATCOLL.OS.Win32.Process; use GNATCOLL.OS.Win32.Process;
with GNATCOLL.OS.Win32; use GNATCOLL.OS.Win32;

separate (GNATCOLL.OS.Process)
function State (H : Process_Handle) return Process_State
is
   WHandle       : constant GNATCOLL.OS.Win32.HANDLE :=
      GNATCOLL.OS.Win32.HANDLE (H);
   Status        : NTSTATUS;
   Process_Info  : aliased PROCESS_BASIC_INFORMATION;
   Result_Length : ULONG;
begin
   Status := NtQueryInformationProcess
      (WHandle,
       ProcessBasicInformation,
       Process_Info'Address,
       PROCESS_BASIC_INFORMATION'Size / 8,
       Result_Length);

   if not Is_Success (Status) then
      return TERMINATED;
   elsif Process_Info.ExitStatus = 259 then
      return RUNNING;
   else
      return WAITABLE;
   end if;
end State;
